@@ -93,6 +93,7 @@
 #define MUSE_TAG_PIXTABLE_REDUCED "PIXTABLE_REDUCED"
 #define MUSE_TAG_OBJECT_RESAMPLED "OBJECT_RESAMPLED"
 #define MUSE_TAG_OFFSET_LIST      "OFFSET_LIST"
+#define MUSE_TAG_SOURCE_LIST      "SOURCE_LIST"
 /* tags used by the muse_qi_mask recipe */
 #define MUSE_TAG_MASK_IMAGE  "MASK_IMAGE"
 /* tags from detmon used by the muse_lingain recipe */
@@ -290,6 +291,7 @@
 #define QC_GEO_SMOOTH_NY    QC_GEO_PREFIX"SMOOTH NY"
 #define QC_GEO_SMOOTH_NA    QC_GEO_PREFIX"SMOOTH NANGLE"
 #define QC_GEO_SMOOTH_NW    QC_GEO_PREFIX"SMOOTH NWIDTH"
+#define QC_GEO_TABLE_NBAD   QC_GEO_PREFIX"TABLE NBAD"
 
 /* Twilight QC headers */
 #define QC_TWILIGHTm_PREFIXi       "ESO QC TWILIGHT%hhu INPUT%u"
@@ -333,6 +335,7 @@
 #define QC_ALIGN_BKG_MEDIAN       "ESO QC EXPALIGN BKG MEDIAN"
 #define QC_ALIGN_BKG_MAD          "ESO QC EXPALIGN BKG MAD"
 #define QC_ALIGN_THRESHOLD        "ESO QC EXPALIGN THRESHOLD"
+#define QC_ALIGN_SRC_POSITIONS    "ESO QC EXPALIGN SRC POSITIONS"
 #define QC_ALIGN_NDET             "ESO QC EXPALIGN NDET"
 #define QC_ALIGN_NDETi            "ESO QC EXPALIGN NDET%d"
 #define QC_ALIGN_NMATCHi          "ESO QC EXPALIGN NMATCH%d"
