@@ -2,7 +2,7 @@
 /* vim:set sw=2 sts=2 et cin: */
 /*
  * This file is part of the MUSE Instrument Pipeline
- * Copyright (C) 2005-2014 European Southern Observatory
+ * Copyright (C) 2005-2016 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,7 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -35,12 +36,12 @@
 #include "muse_processing.h"
 #include "muse_instrument.h"
 
+#include "muse_datacube.h"
 #include "muse_dfs.h"
+#include "muse_idp.h"
 #include "muse_pfits.h"
 #include "muse_lsf.h"
-#include "muse_datacube.h"
 #include "muse_utils.h"
-#include "muse_idp.h"
 
 /*----------------------------------------------------------------------------*
  *                             Debugging Macros                               *
@@ -207,31 +208,47 @@ muse_processing_append_used(muse_processing *aProcessing,
                   cpl_frame_get_tag(aFrame), cpl_frame_get_filename(aFrame));
   }
 #endif
-  /* check, if this frame is already listed */
-  const char *fn = cpl_frame_get_filename(aFrame),
-             *tag = cpl_frame_get_tag(aFrame);
-  cpl_size iframe, nframes = cpl_frameset_get_size(aProcessing->usedframes);
-  for (iframe = 0;
-       (iframe < nframes) && fn && tag; /* only check with valid info */
-       iframe++) {
-    cpl_frame *frame = cpl_frameset_get_position(aProcessing->usedframes, iframe);
-    const char *fn2 = cpl_frame_get_filename(frame),
-               *tag2 = cpl_frame_get_tag(frame);
-    if (fn2 && !strncmp(fn, fn2, strlen(fn) + 1) &&
-        tag2 && !strncmp(tag, tag2, strlen(tag) + 1)) {
-      if (!aDuplicate) { /* destroy frame, assuming it was duplicated elsewhere */
-        cpl_frame_delete(aFrame);
+  /* check, if this frame is already listed   *
+   * (this has to be locked by other threads) */
+  cpl_boolean doreturn = CPL_FALSE;
+#pragma omp critical(muse_processing_used_frames)
+  {
+    const char *fn = cpl_frame_get_filename(aFrame),
+        *tag = cpl_frame_get_tag(aFrame);
+    cpl_size iframe, nframes = cpl_frameset_get_size(aProcessing->usedframes);
+    for (iframe = 0;
+        (iframe < nframes) && fn && tag; /* only check with valid info */
+        iframe++) {
+      cpl_frame *frame = cpl_frameset_get_position(aProcessing->usedframes, iframe);
+      const char *fn2 = cpl_frame_get_filename(frame),
+          *tag2 = cpl_frame_get_tag(frame);
+      if (fn2 && !strncmp(fn, fn2, strlen(fn) + 1) &&
+          tag2 && !strncmp(tag, tag2, strlen(tag) + 1)) {
+        if (!aDuplicate) { /* destroy frame, assuming it was duplicated elsewhere */
+          cpl_frame_delete(aFrame);
+        }
+        doreturn = CPL_TRUE; /* already listed, mark for early return */
+        break;
       }
-      return;
-    }
-  } /* for iframe (all usedframes) */
-  cpl_frame_set_group(aFrame, aGroup);
-  if (aDuplicate) {
-    cpl_frameset_insert(aProcessing->usedframes, cpl_frame_duplicate(aFrame));
-  } else {
-    cpl_frameset_insert(aProcessing->usedframes, aFrame);
+    } /* for iframe (all usedframes) */
+  } /* end of pragma omp */
+
+  if (doreturn) {
+    return; /* frame is already in the usedframes set, so return */
   }
-}
+  /* need to store this frame, so set a group */
+  cpl_frame_set_group(aFrame, aGroup);
+
+  /* actual inserting the frame has to be guarded against other threads */
+#pragma omp critical(muse_processing_used_frames)
+  {
+    if (aDuplicate) {
+      cpl_frameset_insert(aProcessing->usedframes, cpl_frame_duplicate(aFrame));
+    } else {
+      cpl_frameset_insert(aProcessing->usedframes, aFrame);
+    }
+  } /* end of pragma omp */
+} /* muse_processing_append_used() */
 
 /*---------------------------------------------------------------------------*/
 /**
@@ -331,6 +348,8 @@ muse_processing_setup_header(muse_processing *aProcessing,
 #endif
 
   char *pkgstring = cpl_sprintf("%s/%s", PACKAGE, PACKAGE_VERSION);
+
+#pragma omp critical(muse_processing_dfs_setup_header)
   if (cpl_dfs_setup_product_header(aHeader, aFrame, fset,
                                    aProcessing->parameters, aProcessing->name,
                                    pkgstring, MUSE_PRO_DID, NULL)
@@ -478,6 +497,7 @@ muse_processing_save_image(muse_processing *aProcessing, int aIFU,
   cpl_msg_info(__func__, "Saving image as %s", cpl_frame_get_filename(frame));
   int r = muse_image_save(aImage, cpl_frame_get_filename(frame));
   if (r == CPL_ERROR_NONE) {
+#pragma omp critical(muse_processing_output_frames)
     cpl_frameset_insert(aProcessing->outframes, frame);
   } else {
     cpl_frame_delete(frame);
@@ -528,7 +548,7 @@ muse_processing_save_cube(muse_processing *aProcessing, int aIFU,
   if (aType == MUSE_CUBE_TYPE_SDP) {
     cpl_errorstate status = cpl_errorstate_get();
     muse_idp_properties *properties =
-        muse_idp_properties_collect(aProcessing, (muse_datacube *)aCube, aTag);
+      muse_idp_properties_collect(aProcessing, (muse_datacube *)aCube, aTag);
     if (properties) {
       muse_idp_properties_update(((muse_datacube *)aCube)->header, properties);
     }
@@ -554,6 +574,7 @@ muse_processing_save_cube(muse_processing *aProcessing, int aIFU,
                             cpl_frame_get_filename(frame));
   }
   if (rc == CPL_ERROR_NONE) {
+#pragma omp critical(muse_processing_output_frames)
     cpl_frameset_insert(aProcessing->outframes, frame);
   } else {
     cpl_frame_delete(frame);
@@ -561,6 +582,100 @@ muse_processing_save_cube(muse_processing *aProcessing, int aIFU,
   return rc;
 } /* muse_processing_save_cube() */
 
+/*---------------------------------------------------------------------------*/
+/**
+  @brief  Load a CPL table according to its tag and IFU/channel number.
+  @param  aProcessing   the processing structure, includes the input frames list
+  @param  aTag          the tag
+  @param  aIFU          the IFU/channel number
+  @return the cpl_table * or NULL if an error occurred.
+
+  This function tries to find an extension that is specific for the given IFU
+  number, and loads that, if available. If not, it is assumed that either the
+  table is a general one applicable for all IFUs or only contains data for this
+  one IFU.
+
+  In effect, it uses @ref muse_processing_load_table() and strips off the header
+  component.
+
+  @error{set CPL_ERROR_NULL_INPUT\, return NULL, aProcessing is NULL}
+  @error{propagate error code\, return NULL, muse_processing_load_table() fails}
+ */
+/*---------------------------------------------------------------------------*/
+cpl_table *
+muse_processing_load_ctable(muse_processing *aProcessing, const char *aTag,
+                            unsigned char aIFU)
+{
+  cpl_ensure(aProcessing, CPL_ERROR_NULL_INPUT, NULL);
+
+  /* load the table, with header */
+  muse_table *table = muse_processing_load_table(aProcessing, aTag, aIFU);
+  if (!table) {
+    return NULL;
+  }
+  /* strip the header part to return a "plain" CPL table */
+  cpl_table *ctable = table->table;
+  table->table = NULL;
+  muse_table_delete(table);
+
+  return ctable;
+} /* muse_processing_load_ctable() */
+
+/*---------------------------------------------------------------------------*/
+/**
+  @brief  Load a MUSE table according to its tag and IFU/channel number.
+  @param  aProcessing   the processing structure, includes the input frames list
+  @param  aTag          the tag
+  @param  aIFU          the IFU/channel number
+  @return the muse_table * or NULL if an error occurred.
+
+  This function tries to find an extension that is specific for the given IFU
+  number, and loads that, if available. If not, it is assumed that either the
+  table is a general one applicable for all IFUs or only contains data for this
+  one IFU.
+
+  The header component of the returned MUSE table is always loaded from the
+  primary FITS extension.
+
+  @error{set CPL_ERROR_NULL_INPUT\, return NULL, aProcessing is NULL}
+ */
+/*---------------------------------------------------------------------------*/
+muse_table *
+muse_processing_load_table(muse_processing *aProcessing, const char *aTag,
+                           unsigned char aIFU)
+{
+  cpl_ensure(aProcessing, CPL_ERROR_NULL_INPUT, NULL);
+
+  /* find the (first) table that matches the given tag */
+  cpl_frame *frame = muse_frameset_find_master(aProcessing->inframes, aTag,
+                                               aIFU);
+  if (!frame) {
+    if (aIFU > 0) {
+      cpl_msg_debug(__func__, "No table found for tag %s and IFU %hhu", aTag,
+                    aIFU);
+    } else {
+      /* swallow the IFU part, if it's not a real IFU number */
+      cpl_msg_debug(__func__, "No table found for tag %s", aTag);
+    }
+    return NULL;
+  }
+
+  /* try to load the extension table for the given IFU, then just return  */
+  cpl_errorstate state = cpl_errorstate_get();
+  muse_table *table = muse_table_load(cpl_frame_get_filename(frame), aIFU);
+  if (!cpl_errorstate_is_equal(state)) {
+    cpl_error_set(__func__, cpl_error_get_code());
+    muse_table_delete(table);
+    return NULL;
+  }
+
+  /* tables are calibrations of some sort, so append them as such into        *
+   * the used frames for the FITS header of the output product, if they exist */
+  muse_processing_append_used(aProcessing, frame, CPL_FRAME_GROUP_CALIB, 0);
+
+  return table;
+} /* muse_processing_load_table() */
+
 /*---------------------------------------------------------------------------*/
 /**
   @brief  Save a computed table to disk.
@@ -570,8 +685,8 @@ muse_processing_save_cube(muse_processing *aProcessing, int aIFU,
   @param  aHeader       the custom properties to be included in the header
                         (for aType == MUSE_TABLE_TYPE_CPL)
   @param  aTag          tag to apply for output frames, used as prefix
-  @param  aType         type of table to save
-                        (MUSE_TABLE_TYPE_CPL or MUSE_TABLE_TYPE_PIXTABLE)
+  @param  aType         type of table to save (MUSE_TABLE_TYPE_CPL,
+                        MUSE_TABLE_TYPE_MUSE, or MUSE_TABLE_TYPE_PIXTABLE)
   @return CPL_ERROR_NONE or the relevant cpl_error_code on error
 
   Save a table to disk. The filename consists on the prefix, a "-"
@@ -589,14 +704,15 @@ muse_processing_save_table(muse_processing *aProcessing, int aIFU,
                            const char *aTag, muse_table_type aType)
 {
   cpl_ensure_code(aProcessing && aTable && aTag, CPL_ERROR_NULL_INPUT);
-  cpl_ensure_code(aType == MUSE_TABLE_TYPE_CPL || aType == MUSE_TABLE_TYPE_PIXTABLE,
-                  CPL_ERROR_ILLEGAL_INPUT);
+  cpl_ensure_code(aType <= MUSE_TABLE_TYPE_MUSE, CPL_ERROR_ILLEGAL_INPUT);
 
   /* for plain tables, the separate header is needed */
   cpl_propertylist *header;
   if (aType == MUSE_TABLE_TYPE_CPL) {
     cpl_ensure_code(aHeader, CPL_ERROR_NULL_INPUT);
     header = aHeader;
+  } else if (aType == MUSE_TABLE_TYPE_MUSE) {
+    header = ((muse_table *)aTable)->header;
   } else {
     header = ((muse_pixtable *)aTable)->header;
   }
@@ -613,11 +729,14 @@ muse_processing_save_table(muse_processing *aProcessing, int aIFU,
     if (nrow < 1) {
       cpl_msg_warning(__func__, "Table saved as \"%s\" has no rows!", aTag);
     }
+  } else if (aType == MUSE_TABLE_TYPE_MUSE) {
+    rc = muse_table_save((muse_table *)aTable, cpl_frame_get_filename(frame));
   } else {
     rc = muse_pixtable_save((muse_pixtable *)aTable,
                             cpl_frame_get_filename(frame));
   }
   if (rc == CPL_ERROR_NONE) {
+#pragma omp critical(muse_processing_output_frames)
     cpl_frameset_insert(aProcessing->outframes, frame);
   } else {
     cpl_msg_error(__func__, "Saving %stable failed: %s",
@@ -660,6 +779,7 @@ muse_processing_save_cimage(muse_processing *aProcessing, int aIFU,
   int r = cpl_image_save(aImage, cpl_frame_get_filename(frame),
                          CPL_TYPE_UNSPECIFIED, aHeader, CPL_IO_CREATE);
   if (r == CPL_ERROR_NONE) {
+#pragma omp critical(muse_processing_output_frames)
     cpl_frameset_insert(aProcessing->outframes, frame);
   } else {
     cpl_msg_error(__func__, "Saving image failed: %s", cpl_error_get_message());
@@ -668,6 +788,46 @@ muse_processing_save_cimage(muse_processing *aProcessing, int aIFU,
   return r;
 }
 
+/*---------------------------------------------------------------------------*/
+/**
+  @brief  load a header according to its tag
+  @param  aProcessing   the processing structure, includes the input frames list
+  @param  aTag          the tag
+  @return the cpl_propertylist * or NULL if an error occurred.
+
+  This function tries to find the frame in the input frameset and loads the
+  propertylist from it.
+
+  @error{set CPL_ERROR_NULL_INPUT\, return NULL, aProcessing is NULL}
+ */
+/*---------------------------------------------------------------------------*/
+cpl_propertylist *
+muse_processing_load_header(muse_processing *aProcessing, const char *aTag)
+{
+  cpl_ensure(aProcessing, CPL_ERROR_NULL_INPUT, NULL);
+
+  cpl_frame *frame = muse_frameset_find_master(aProcessing->inframes, aTag, 0);
+  if (!frame) {
+    cpl_msg_debug(__func__, "No propertylist found for tag %s", aTag);
+    return NULL;
+  }
+
+  const char *filename = cpl_frame_get_filename(frame);
+  cpl_propertylist *pl = cpl_propertylist_load(filename, 0);
+  if (!pl) {
+    cpl_msg_info(__func__, "loading %s from file %s failed: %s", aTag,
+                 filename, cpl_error_get_message());
+    cpl_frame_delete(frame);
+    return NULL;
+  }
+  cpl_msg_info(__func__, "loaded %s from file \"%s\"", aTag, filename);
+  /* property lists are calibrations of some sort, so append them as such into *
+   * the used frames for the FITS header of the output product, if they exist  */
+  muse_processing_append_used(aProcessing, frame, CPL_FRAME_GROUP_CALIB, 0);
+
+  return pl;
+} /* muse_processing_load_header() */
+
 /*---------------------------------------------------------------------------*/
 /**
   @brief Save a FITS header to disk.
@@ -699,13 +859,53 @@ muse_processing_save_header(muse_processing *aProcessing, int aIFU,
                                             cpl_frame_get_filename(frame),
                                             CPL_IO_CREATE);
   if (rc == CPL_ERROR_NONE) {
+#pragma omp critical(muse_processing_output_frames)
     cpl_frameset_insert(aProcessing->outframes, frame);
   } else {
     cpl_msg_error(__func__, "Saving header failed: %s", cpl_error_get_message());
     cpl_frame_delete(frame);
   }
   return rc;
-}
+} /* muse_processing_save_header() */
+
+/*---------------------------------------------------------------------------*/
+/**
+  @brief  Load a mask file and its FITS header
+  @param  aProcessing   the processing structure, includes the input frames list
+  @param  aTag          the tag
+  @return a new <tt><b>muse_mask *</b></tt> or NULL on error
+  @remark The new mask has to be deallocated using
+          <tt><b>muse_mask_delete()</b></tt>.
+  @remark Only FITS keywords from the primary FITS header will be loaded.
+
+  @error{return NULL\, propagate CPL error code,
+         muse_frameset_find() fails to find a frame of the given tag}
+  @error{return NULL\, propagate CPL error code, muse_mask_load() returns NULL}
+ */
+/*---------------------------------------------------------------------------*/
+muse_mask *
+muse_processing_load_mask(muse_processing *aProcessing, const char *aTag)
+{
+  cpl_frameset *frames = muse_frameset_find(aProcessing->inframes,
+                                            aTag, 0, CPL_FALSE);
+  if (frames == NULL || cpl_frameset_get_size(frames) <= 0) {
+    cpl_frameset_delete(frames);
+    return NULL;
+  }
+  cpl_frame *frame = cpl_frameset_get_position(frames, 0);
+  muse_mask *mask = muse_mask_load(cpl_frame_get_filename(frame));
+  if (!mask) {
+    cpl_msg_warning(__func__, "loading mask \"%s\" failed!",
+                    cpl_frame_get_filename(frame));
+    cpl_frameset_delete(frames);
+    return NULL;
+  }
+  cpl_msg_info(__func__, "using mask \"%s\" (%"CPL_SIZE_FORMAT" pixels)",
+               cpl_frame_get_filename(frame), cpl_mask_count(mask->mask));
+  muse_processing_append_used(aProcessing, frame, CPL_FRAME_GROUP_CALIB, 1);
+  cpl_frameset_delete(frames);
+  return mask;
+} /* muse_processing_load_mask() */
 
 /*---------------------------------------------------------------------------*/
 /**
@@ -736,6 +936,7 @@ muse_processing_save_mask(muse_processing *aProcessing, int aIFU,
   cpl_msg_info(__func__, "Saving mask as %s", cpl_frame_get_filename(frame));
   int r = muse_mask_save(aMask, cpl_frame_get_filename(frame));
   if (r == CPL_ERROR_NONE) {
+#pragma omp critical(muse_processing_output_frames)
     cpl_frameset_insert(aProcessing->outframes, frame);
   } else {
     cpl_frame_delete(frame);
@@ -1040,43 +1241,4 @@ muse_processing_sort_exposures(muse_processing *aProcessing)
   return exptable;
 } /* muse_processing_sort_exposures() */
 
-/*---------------------------------------------------------------------------*/
-/**
-  @brief  Load a mask file and its FITS header
-  @param  aProcessing   the processing structure, includes the input frames list
-  @param  aTag          the tag
-  @return a new <tt><b>muse_mask *</b></tt> or NULL on error
-  @remark The new mask has to be deallocated using
-          <tt><b>muse_mask_delete()</b></tt>.
-  @remark Only FITS keywords from the primary FITS header will be loaded.
-
-  @error{return NULL\, propagate CPL error code,
-         muse_frameset_find() fails to find a frame of the given tag}
-  @error{return NULL\, propagate CPL error code, muse_mask_load() returns NULL}
- */
-/*---------------------------------------------------------------------------*/
-muse_mask *
-muse_processing_mask_load(muse_processing *aProcessing, const char *aTag)
-{
-  cpl_frameset *frames = muse_frameset_find(aProcessing->inframes,
-                                            aTag, 0, CPL_FALSE);
-  if (frames == NULL || cpl_frameset_get_size(frames) <= 0) {
-    cpl_frameset_delete(frames);
-    return NULL;
-  }
-  cpl_frame *frame = cpl_frameset_get_position(frames, 0);
-  muse_mask *mask = muse_mask_load(cpl_frame_get_filename(frame));
-  if (!mask) {
-    cpl_msg_warning(__func__, "loading mask \"%s\" failed!",
-                    cpl_frame_get_filename(frame));
-    cpl_frameset_delete(frames);
-    return NULL;
-  }
-  cpl_msg_info(__func__, "using mask \"%s\" (%"CPL_SIZE_FORMAT" pixels)",
-               cpl_frame_get_filename(frame), cpl_mask_count(mask->mask));
-  muse_processing_append_used(aProcessing, frame, CPL_FRAME_GROUP_CALIB, 1);
-  cpl_frameset_delete(frames);
-  return mask;
-}
-
 /**@}*/
