@@ -270,7 +270,7 @@ muse_wcs_centroid_stars(muse_image *aImage, float aSigma,
     /* Remove the detection if the FWHM could not be computed in both    *
      * axes, since this likely points to an object on the border or with *
      * some other problem which should not be relied on for centroiding. */
-    if (xwaper < 0 || ywaper < 0) {
+    if (xwaper < 0 || ywaper < 0 || xwaper > 100. || ywaper > 100.) {
       cpl_msg_debug(__func__, "FWHM computation unsuccessful at %f,%f, result "
                     "was %.3f,%.3f", xc, yc, xwaper, ywaper);
       cpl_table_select_row(detections, idet);
@@ -434,7 +434,8 @@ muse_wcs_centroid_stars(muse_image *aImage, float aSigma,
     cpl_table_set(detections, "FLUX", idet, flux);
 
     if (cpl_errorstate_is_equal(state) && xw > 0 && yw > 0 &&
-        xcen >= 1 && xcen <= nx && ycen >= 1 && ycen <= ny) {
+        xcen >= 1 && xcen <= nx && ycen >= 1 && ycen <= ny &&
+        xw < 100. && yw < 100. && xerr < 100. && yerr < 100.) {
       continue;
     }
     /* some error occurred, so mark this entry for removal */
@@ -485,7 +486,7 @@ muse_wcs_centroid_stars(muse_image *aImage, float aSigma,
   @error{return CPL_ERROR_ILLEGAL_INPUT,
          an unknown centroiding type was requested}
   @error{output warning\, set CPL_ERROR_UNSUPPORTED_MODE\, but continue processing,
-         input pixel table was not corrected for DAR}
+         input WFM pixel table was not corrected for DAR}
   @error{propagate error of muse_resampling_cube(), cube could not be created}
   @error{propagate error, image could not be created from cube}
   @error{propagate error from muse_wcs_centroid_stars(),
@@ -522,7 +523,8 @@ muse_wcs_locate_sources(muse_pixtable *aPixtable, float aSigma,
   } else if (cpl_propertylist_has(aPixtable->header, MUSE_HDR_PT_DAR_CORR)) {
     darcorrected = CPL_TRUE;
   }
-  if (!darcorrected) {
+  cpl_boolean isNFM = muse_pfits_get_mode(aPixtable->header) == MUSE_MODE_NFM_AO_N;
+  if (!darcorrected && !isNFM) {
     const char *message = "Correction for differential atmospheric refraction "
                           "was not applied! Deriving astrometric correction "
                           "from such data is unlikely to give good results!";
@@ -994,8 +996,10 @@ muse_wcs_optimize_solution(muse_wcs_object *aWCSObj, float aDetSigma,
 {
   cpl_ensure_code(aWCSObj && aWCSObj->cube, CPL_ERROR_NULL_INPUT);
   /* check that the cube contains the detection image */
-  cpl_ensure_code(!strncmp(cpl_array_get_string(aWCSObj->cube->recnames, 1),
-                         MUSE_WCS_DETIMAGE_EXTNAME, strlen(MUSE_WCS_DETIMAGE_EXTNAME) + 1),
+  cpl_ensure_code(cpl_array_get_size(aWCSObj->cube->recnames) > 1 &&
+                  !strncmp(cpl_array_get_string(aWCSObj->cube->recnames, 1),
+                           MUSE_WCS_DETIMAGE_EXTNAME,
+                           strlen(MUSE_WCS_DETIMAGE_EXTNAME) + 1),
                   CPL_ERROR_DATA_NOT_FOUND);
   cpl_ensure_code(aDetSigma < 0., CPL_ERROR_ILLEGAL_INPUT);
   switch (aCentroid) {
