@@ -283,10 +283,12 @@ class DataPlotterManager(object):
             reflex.RecipeParameter(recipe=self.recipeName,
                 displayName="weight", group="Offset Calculation",
                 description="Use weighting"),
-
             reflex.RecipeParameter(recipe=self.recipeName,
                 displayName="fwhm", group="Source Detection",
                 description="FWHM of the convolution filter"),
+            reflex.RecipeParameter(recipe=self.recipeName,
+                displayName="bpixdistance", group="Source Detection",
+                description="Minimum distance from image border [pixels]"),
             reflex.RecipeParameter(recipe=self.recipeName,
                 displayName="threshold", group="Source Detection",
                 description="Initial threshold for detecting point sources"),
