@@ -860,7 +860,12 @@ int main(int argc, char **argv)
                                         "ESO QC STANDARD ZP R");
   /* the INM apparently simulated a nice throughput of 27.3% in the R-band */
   cpl_test(isfinite(zp));
+#if CPL_VERSION_CODE < 459009 /* before than CPL v7.1.1b3 */
   cpl_test_rel(zp, 1.41028, 1e-5);
+#else /* with the changes to cpl_fit_lvmq() the values of the Moffat   *
+       * extraction seem to have changed enough to require a new value */
+  cpl_test_rel(zp, 1.41059, 1e-5);
+#endif
   rc = muse_flux_compute_qc_zp(fluxobj, cousinsr, NULL);
   cpl_test(cpl_errorstate_is_equal(state) && rc == CPL_ERROR_NONE);
   float zp2 = cpl_propertylist_get_float(fluxobj->response->header,
